@@ -1,6 +1,6 @@
 /*
- * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2015 Hiroyuki Yamamoto and the Claws Mail team
+ * Claws Mail -- a GTK+ based, lightweight, and fast e-mail client
+ * Copyright (C) 1999-2021 the Claws Mail team and Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -87,6 +87,7 @@ static struct EditAccount {
 	GtkWidget *window;
 	GtkWidget *list_view;
 	GtkWidget *close_btn;
+	GtkWidget *label;
 } edit_account;
 
 static void account_edit_create		(void);
@@ -701,13 +702,15 @@ static void account_size_allocate_cb(GtkWidget *widget,
 
 	prefs_common.accountswin_width = allocation->width;
 	prefs_common.accountswin_height = allocation->height;
+
+	gtk_widget_set_size_request(GTK_WIDGET(edit_account.label), 
+				    prefs_common.accountswin_width-8, -1);
 }
 
 static void account_edit_create(void)
 {
 	GtkWidget *window;
 	GtkWidget *vbox;
-	GtkWidget *label;
 	GtkWidget *hbox;
 	GtkWidget *scrolledwin;
 	GtkWidget *list_view;
@@ -748,16 +751,14 @@ static void account_edit_create(void)
 	gtk_widget_show (hbox);
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
-	label = gtk_label_new
+	edit_account.label = gtk_label_new
 		(_("Using 'Get Mail' will retrieve messages from your Accounts "
 		   "in the order given, the checkbox indicates which accounts "
 		   "will be included. Bold text indicates the default account."));
-	gtk_widget_show (label);
-	gtk_widget_set_size_request(GTK_WIDGET(label), 
-				    prefs_common.accountswin_width-8, -1);
-	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 4);
-	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
-	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
+	gtk_widget_show (edit_account.label);
+	gtk_box_pack_start (GTK_BOX (hbox), edit_account.label, FALSE, FALSE, 4);
+	gtk_label_set_justify(GTK_LABEL(edit_account.label), GTK_JUSTIFY_LEFT);
+	gtk_label_set_line_wrap(GTK_LABEL(edit_account.label), TRUE);
 
 	hbox = gtk_hbox_new (FALSE, 8);
 	gtk_widget_show (hbox);
@@ -784,36 +785,48 @@ static void account_edit_create(void)
 	gtk_box_pack_start (GTK_BOX (vbox2), add_btn, FALSE, FALSE, 4);
 	g_signal_connect (G_OBJECT(add_btn), "clicked",
 			  G_CALLBACK (account_add), NULL);
+	CLAWS_SET_TIP(add_btn,
+			_("Create a new account"));
 
 	edit_btn = gtk_button_new_from_stock (GTK_STOCK_EDIT);
 	gtk_widget_show (edit_btn);
 	gtk_box_pack_start (GTK_BOX (vbox2), edit_btn, FALSE, FALSE, 4);
 	g_signal_connect (G_OBJECT(edit_btn), "clicked",
 			  G_CALLBACK (account_edit_prefs), NULL);
+	CLAWS_SET_TIP(edit_btn,
+			_("Edit preferences for the selected account"));
 
 	del_btn = gtk_button_new_from_stock(GTK_STOCK_DELETE);
 	gtk_widget_show (del_btn);
 	gtk_box_pack_start (GTK_BOX (vbox2), del_btn, FALSE, FALSE, 4);
 	g_signal_connect (G_OBJECT(del_btn), "clicked",
 			  G_CALLBACK (account_delete), NULL);
+	CLAWS_SET_TIP(del_btn,
+			_("Delete the selected account from the list"));
 
 	clone_btn = gtk_button_new_from_stock(GTK_STOCK_COPY);
 	gtk_widget_show (clone_btn);
 	gtk_box_pack_start (GTK_BOX (vbox2), clone_btn, FALSE, FALSE, 4);
 	g_signal_connect(G_OBJECT(clone_btn), "clicked",
 			 G_CALLBACK(account_clone), NULL);
-	
+	CLAWS_SET_TIP(clone_btn,
+			_("Create a new copy of the selected account"));
+
 	down_btn = gtk_button_new_from_stock(GTK_STOCK_GO_DOWN);
 	gtk_widget_show (down_btn);
 	gtk_box_pack_end (GTK_BOX (vbox2), down_btn, FALSE, FALSE, 4);
 	g_signal_connect (G_OBJECT(down_btn), "clicked",
 			  G_CALLBACK (account_down), NULL);
+	CLAWS_SET_TIP(down_btn,
+			_("Move the selected account down"));
 
 	up_btn = gtk_button_new_from_stock(GTK_STOCK_GO_UP);
 	gtk_widget_show (up_btn);
 	gtk_box_pack_end (GTK_BOX (vbox2), up_btn, FALSE, FALSE, 4);
 	g_signal_connect (G_OBJECT(up_btn), "clicked",
 			  G_CALLBACK (account_up), NULL);
+	CLAWS_SET_TIP(up_btn,
+			_("Move the selected account up"));
 
 	hbox = gtk_hbox_new (FALSE, 8);
 	gtk_widget_show (hbox);
@@ -1507,17 +1520,17 @@ static void account_list_store_insert_account_item(GtkListStore *list_store,
 #ifdef USE_GNUTLS
 	protocol = ac_prefs->protocol == A_POP3 ?
 		  (ac_prefs->ssl_pop == SSL_TUNNEL ?
-		   "POP (SSL/TLS)" :
+		   "POP (TLS)" :
 		   ac_prefs->ssl_pop == SSL_STARTTLS ?
 		   "POP (STARTTLS)" : "POP") :
 		   ac_prefs->protocol == A_IMAP4 ?
 		  (ac_prefs->ssl_imap == SSL_TUNNEL ?
-		   "IMAP (SSL/TLS)" :
+		   "IMAP (TLS)" :
 		   ac_prefs->ssl_imap == SSL_STARTTLS ?
 		   "IMAP (STARTTLS)" : "IMAP") :
 		   ac_prefs->protocol == A_NNTP ?
 		  (ac_prefs->ssl_nntp == SSL_TUNNEL ?
-		   "NNTP (SSL/TLS)" : "NNTP") :
+		   "NNTP (TLS)" : "NNTP") :
 		   ac_prefs->protocol == A_LOCAL ? "Local" :
 		   ac_prefs->protocol == A_NONE ?  "SMTP" : "-";
 #else
