@@ -959,9 +959,12 @@ static ChildInfo *fork_child(gchar *cmd, const gchar *msg_str,
 	ret_str = g_locale_from_utf8(cmd, strlen(cmd),
 				     &by_read, &by_written,
 				     NULL);
-	if (!ret_str || !by_written)
+	if (!ret_str || !by_written) {
+		if (ret_str)
+			g_free(ret_str);
 		ret_str = g_strdup(cmd);
-
+	}
+    
 	trim_cmd = ret_str;
 
 	while (g_ascii_isspace(trim_cmd[0]))
@@ -1042,11 +1045,13 @@ static ChildInfo *fork_child(gchar *cmd, const gchar *msg_str,
 		int r;
 		ret_str = g_locale_from_utf8(msg_str, strlen(msg_str),
 					     &by_read, &by_written, NULL);
-		if (ret_str && by_written) {
+		if (ret_str && by_written)
 			r = write(chld_in, ret_str, strlen(ret_str));
-			g_free(ret_str);
-		} else
+		else
 			r = write(chld_in, msg_str, strlen(msg_str));
+		if (ret_str)
+			g_free(ret_str);
+
 		if (!(children->action_type &
 		      (ACTION_USER_IN | ACTION_USER_HIDDEN_IN)))
 			r = close(chld_in);
@@ -1144,7 +1149,7 @@ static gint io_dialog_key_pressed_cb(GtkWidget *widget, GdkEventKey *event,
 {
 	if (event && (event->keyval == GDK_KEY_Escape ||
 		      event->keyval == GDK_KEY_Return ||
-			  event->keyval == GDK_KEY_KP_Enter))
+		      event->keyval == GDK_KEY_KP_Enter))
 		hide_io_dialog_cb(widget, data);
 	return TRUE;
 }
@@ -1515,9 +1520,12 @@ static void catch_input(gpointer data, gint source, GIOCondition cond)
 				       0, -1);
 	ret_str = g_locale_from_utf8(input, strlen(input), &by_read,
 				     &by_written, NULL);
-	if (ret_str && by_written) {
-		g_free(input);
-		input = ret_str;
+	if (ret_str) {
+		if (by_written) {
+			g_free(input);
+			input = ret_str;
+		} else
+			g_free(ret_str);
 	}
 
 	len = strlen(input);
@@ -1568,18 +1576,18 @@ static void catch_output(gpointer data, gint source, GIOCondition cond)
 			gchar *ret_str;
 
 			c = read(source, buf, sizeof(buf) - 1);
-			if (c == 0)
+			if (c <= 0)
 				break;
 
+			buf[c] = 0;
 			ret_str = g_locale_to_utf8
-				(buf, c - 1, &bytes_read, &bytes_written, NULL);
-			if (ret_str && bytes_written > 0) {
-				gtk_text_buffer_insert
-					(textbuf, &iter, ret_str,
-					 -1);
+				(buf, c, &bytes_read, &bytes_written, NULL);
+			if (ret_str && bytes_written > 0)
+				gtk_text_buffer_insert(textbuf, &iter, ret_str, -1);
+			else
+				gtk_text_buffer_insert(textbuf, &iter, buf, c);
+			if (ret_str)
 				g_free(ret_str);
-			} else
-				gtk_text_buffer_insert(textbuf, &iter, buf, c - 1);
 		}
 
 		if (child_info->children->is_selection) {
@@ -1595,12 +1603,15 @@ static void catch_output(gpointer data, gint source, GIOCondition cond)
 			gsize bytes_read = 0, bytes_written = 0;
 			gchar *ret_str;
 
+			buf[c] = 0;
 			ret_str = g_locale_to_utf8
 				(buf, c, &bytes_read, &bytes_written, NULL);
-			if (ret_str && bytes_written > 0) {
-				g_string_append_len
-					(child_info->output, ret_str,
-					 bytes_written);
+			if (ret_str) {
+				if (bytes_written) {
+					g_string_append_len
+						(child_info->output, ret_str,
+						bytes_written);
+				}
 				g_free(ret_str);
 			} else
 				g_string_append_len(child_info->output, buf, c);
@@ -1650,7 +1661,7 @@ static gchar *get_user_string(const gchar *action, ActionType type)
 			(_("Action's user argument"), message, NULL);
 		break;
 	default:
-		g_warning("Unsupported action type %d", type);
+		g_warning("unsupported action type %d", type);
 	}
 
 	return user_str;
