@@ -38,10 +38,6 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 
-#ifdef G_OS_WIN32
-#  include <w32lib.h>
-#endif
-
 #include "mgutils.h"
 #include "utils.h"
 #include "exporthtml.h"
