@@ -1,6 +1,6 @@
 /*
- * Claws Mail -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2020 the Claws Mail team and Hiroyuki Yamamoto
+ * Claws Mail -- a GTK based, lightweight, and fast e-mail client
+ * Copyright (C) 1999-2022 the Claws Mail team and Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -57,7 +57,6 @@
 #include "prefs_common.h"
 #include "manage_window.h"
 #include "manual.h"
-#include "combobox.h"
 
 gboolean gtkut_get_font_size(GtkWidget *widget,
 			     gint *width, gint *height)
@@ -456,7 +455,7 @@ static gboolean gtkut_text_buffer_find(GtkTextBuffer *buffer, const GtkTextIter
 
 	wcs = g_utf8_to_ucs4(str, -1, &items_read, &items_written, &error);
 	if (error != NULL) {
-		g_warning("An error occurred while converting a string from UTF-8 to UCS-4: %s",
+		g_warning("an error occurred while converting a string from UTF-8 to UCS-4: %s",
 			  error->message);
 		g_error_free(error);
 	}
@@ -492,7 +491,7 @@ static gboolean gtkut_text_buffer_find_backward(GtkTextBuffer *buffer,
 
 	wcs = g_utf8_to_ucs4(str, -1, &items_read, &items_written, &error);
 	if (error != NULL) {
-		g_warning("An error occurred while converting a string from UTF-8 to UCS-4: %s",
+		g_warning("an error occurred while converting a string from UTF-8 to UCS-4: %s",
 			  error->message);
 		g_error_free(error);
 	}
@@ -625,8 +624,8 @@ void gtkut_window_popup(GtkWidget *window)
 	cm_return_if_fail(window != NULL);
 	cm_return_if_fail(gdkwin != NULL);
 
-	sx = gdk_screen_width();
-	sy = gdk_screen_height();
+	sx = MAX(1, gdk_screen_width());
+	sy = MAX(1, gdk_screen_height());
 
 	gdk_window_get_origin(gdkwin, &x, &y);
 	new_x = x % sx; if (new_x < 0) new_x = 0;
@@ -649,8 +648,8 @@ void gtkut_widget_get_uposition(GtkWidget *widget, gint *px, gint *py)
 	cm_return_if_fail(widget != NULL);
 	cm_return_if_fail(gdkwin != NULL);
 
-	sx = gdk_screen_width();
-	sy = gdk_screen_height();
+	sx = MAX(1, gdk_screen_width());
+	sy = MAX(1, gdk_screen_height());
 
 	/* gdk_window_get_root_origin ever return *rootwindow*'s position */
 	gdk_window_get_root_origin(gdkwin, &x, &y);
@@ -715,8 +714,8 @@ void gtkut_widget_set_app_icon(GtkWidget *widget)
 	cm_return_if_fail(gtk_widget_get_window(widget) != NULL);
 	if (!icon_list) {
 		GdkPixbuf *icon = NULL, *big_icon = NULL;
-		stock_pixbuf_gdk(STOCK_PIXMAP_CLAWS_MAIL_ICON, &icon);
-		stock_pixbuf_gdk(STOCK_PIXMAP_CLAWS_MAIL_LOGO, &big_icon);
+		priv_pixbuf_gdk(PRIV_PIXMAP_CLAWS_MAIL_ICON, &icon);
+		priv_pixbuf_gdk(PRIV_PIXMAP_CLAWS_MAIL_LOGO, &big_icon);
 		if (icon)
 			icon_list = g_list_append(icon_list, icon);
 		if (big_icon)
@@ -1088,6 +1087,8 @@ GtkWidget *xface_get_from_header(const gchar *o_xface)
 {
 	static gchar *xpm_xface[XPM_XFACE_HEIGHT];
 	static gboolean xpm_xface_init = TRUE;
+	GdkPixbuf *pixbuf;
+	GtkWidget *ret;
 	gchar xface[2048];
 	
 	if (o_xface == NULL)
@@ -1113,8 +1114,11 @@ GtkWidget *xface_get_from_header(const gchar *o_xface)
 
 	create_xpm_from_xface(xpm_xface, xface);
 
-	return gtk_image_new_from_pixbuf(
-		gdk_pixbuf_new_from_xpm_data((const char **)xpm_xface));
+	pixbuf = gdk_pixbuf_new_from_xpm_data((const char **)xpm_xface);
+	ret = gtk_image_new_from_pixbuf(pixbuf);
+	g_object_unref(pixbuf);
+
+	return ret;
 }
 #endif
 
