@@ -1,6 +1,6 @@
 /*
- * Claws Mail -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2016 Hiroyuki Yamamoto and the Claws Mail team
+ * Claws Mail -- a GTK based, lightweight, and fast e-mail client
+ * Copyright (C) 1999-2022 the Claws Mail team and Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -230,7 +230,9 @@ static void open_compose_new		(const gchar	*address,
 
 static void send_queue			(void);
 static void initial_processing		(FolderItem *item, gpointer data);
+#ifndef G_OS_WIN32
 static void quit_signal_handler         (int sig);
+#endif
 static void install_basic_sighandlers   (void);
 #if (defined linux && defined SIGIO)
 static void install_memory_sighandler   (void);
@@ -379,12 +381,15 @@ static gboolean migrate_old_config(const gchar *old_cfg_dir, const gchar *new_cf
 {
 	gchar *message = g_strdup_printf(_("Configuration for %s found.\n"
 			 "Do you want to migrate this configuration?"), oldversion);
-	gchar *message2 = g_strdup_printf(_("\n\nYour Sylpheed filtering rules can be converted by a\n"
-			     "script available at %s."), TOOLS_URI);
 
-	if (!strcmp(oldversion, "Sylpheed"))
-		message = g_strconcat(message, message2, NULL);
-	g_free(message2);
+	if (!strcmp(oldversion, "Sylpheed")) {
+		gchar *message2 = g_strdup_printf(_("\n\nYour Sylpheed filtering rules can be converted by a\n"
+			     "script available at %s."), TOOLS_URI);
+		gchar *tmp = g_strconcat(message, message2, NULL);
+		g_free(message2);
+		g_free(message);
+        message = tmp;
+	}
 
 	gint r = 0;
 	GtkWidget *window = NULL;
@@ -657,7 +662,7 @@ static void sc_session_manager_connect(MainWindow *mainwin)
 			g_free(client_id);
 
 		if (error_string_ret[0] || mainwin->smc_conn == NULL)
-			g_warning ("While connecting to session manager: %s.",
+			g_warning("while connecting to session manager: %s",
 				error_string_ret);
 		else {
 			SmPropValue *vals;
@@ -693,30 +698,59 @@ void main_set_show_at_startup(gboolean show)
 }
 
 #ifdef G_OS_WIN32
-static FILE* win32_debug_fp=NULL;
+static HANDLE win32_debug_log = NULL;
 static guint win32_log_handler_app_id;
 static guint win32_log_handler_glib_id;
 static guint win32_log_handler_gtk_id;
 
-static void win32_print_stdout(const gchar *string)
+static void win32_log_WriteFile(const gchar *string)
 {
-	if (win32_debug_fp) {
-		fprintf(win32_debug_fp, "%s", string);
-		fflush(win32_debug_fp);
+	BOOL ret;
+	DWORD bytes_written;
+
+	ret = WriteFile(win32_debug_log, string, strlen(string), &bytes_written, NULL);
+	if (!ret) {
+		DWORD err = GetLastError();
+		gchar *tmp;
+
+		tmp = g_strdup_printf("Error: WriteFile in failed with error 0x%lx.  Buffer contents:\n%s", err, string);
+		OutputDebugString(tmp);
+		g_free(tmp);
 	}
 }
 
-static void win32_print_stderr(const gchar *string)
+static void win32_print_stdout(const gchar *string)
 {
-	if (win32_debug_fp) {
-		fprintf(win32_debug_fp, "%s", string);
-		fflush(win32_debug_fp);
+	if (win32_debug_log) {
+		win32_log_WriteFile(string);
 	}
 }
 
+GLogWriterOutput win32_log_writer(GLogLevelFlags log_level, const GLogField *fields, gsize n_fields, gpointer user_data)
+{
+	gchar *formatted;
+	gchar *out;
+
+	g_return_val_if_fail(win32_debug_log != NULL, G_LOG_WRITER_UNHANDLED);
+	g_return_val_if_fail(fields != NULL, G_LOG_WRITER_UNHANDLED);
+	g_return_val_if_fail(n_fields > 0, G_LOG_WRITER_UNHANDLED);
+
+	formatted = g_log_writer_format_fields(log_level, fields, n_fields, FALSE);
+	out = g_strdup_printf("%s\n", formatted);
+
+	win32_log_WriteFile(out);
+
+	g_free(formatted);
+	g_free(out);
+
+	return G_LOG_WRITER_HANDLED;
+}
+
 static void win32_log(const gchar *log_domain, GLogLevelFlags log_level, const gchar* message, gpointer user_data)
 {
-	if (win32_debug_fp) {
+	gchar *out;
+
+	if (win32_debug_log) {
 		const gchar* type;
 
 		switch(log_level & G_LOG_LEVEL_MASK)
@@ -742,11 +776,15 @@ static void win32_log(const gchar *log_domain, GLogLevelFlags log_level, const g
 			default:
 				type="N/A";
 		}
+
 		if (log_domain)
-			fprintf(win32_debug_fp, "%s: %s: %s", log_domain, type, message);
+			out = g_strdup_printf("%s: %s: %s", log_domain, type, message);
 		else
-			fprintf(win32_debug_fp, "%s: %s", type, message);
-		fflush(win32_debug_fp);
+			out = g_strdup_printf("%s: %s", type, message);
+
+		win32_log_WriteFile(out);
+
+		g_free(out);
 	}
 }
 
@@ -759,31 +797,45 @@ static void win32_open_log(void)
 		if (rename_force(logfile, oldlogfile) < 0)
 			FILE_OP_ERROR(logfile, "rename");
 	}
-	win32_debug_fp = claws_fopen(logfile, "w");
+
+	win32_debug_log = CreateFile(logfile,
+		GENERIC_WRITE,
+		FILE_SHARE_READ,
+		NULL,
+		CREATE_NEW,
+		FILE_ATTRIBUTE_NORMAL,
+		NULL);
+
+	if (win32_debug_log == INVALID_HANDLE_VALUE) {
+		win32_debug_log = NULL;
+	}
+
 	g_free(logfile);
 	g_free(oldlogfile);
-	if (win32_debug_fp)
-	{
+
+	if (win32_debug_log) {
 		g_set_print_handler(win32_print_stdout);
 		g_set_printerr_handler(win32_print_stdout);
+
 		win32_log_handler_app_id = g_log_set_handler(NULL, G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL
                      | G_LOG_FLAG_RECURSION, win32_log, NULL);
 		win32_log_handler_glib_id = g_log_set_handler("GLib", G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL
                      | G_LOG_FLAG_RECURSION, win32_log, NULL);
 		win32_log_handler_gtk_id = g_log_set_handler("Gtk", G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL
                      | G_LOG_FLAG_RECURSION, win32_log, NULL);
+
+		g_log_set_writer_func(&win32_log_writer, NULL, NULL);
 	}
 }
 
 static void win32_close_log(void)
 {
-	if (win32_debug_fp)
-	{
+	if (win32_debug_log) {
 		g_log_remove_handler("", win32_log_handler_app_id);
 		g_log_remove_handler("GLib", win32_log_handler_glib_id);
 		g_log_remove_handler("Gtk", win32_log_handler_gtk_id);
-		claws_fclose(win32_debug_fp);
-		win32_debug_fp=NULL;
+		CloseHandle(win32_debug_log);
+		win32_debug_log = NULL;
 	}
 }		
 #endif
@@ -969,14 +1021,14 @@ static void install_dbus_status_handler(void)
 			"com.google.code.Awn");
 	dbus_item_hook_id = hooks_register_hook (FOLDER_ITEM_UPDATE_HOOKLIST, dbus_status_update_item_hook, NULL);
 	if (dbus_item_hook_id == HOOK_NONE) {
-		g_warning("Failed to register folder item update hook");
+		g_warning("failed to register folder item update hook");
 		uninstall_dbus_status_handler();
 		return;
 	}
 
 	dbus_folder_hook_id = hooks_register_hook (FOLDER_UPDATE_HOOKLIST, dbus_status_update_folder_hook, NULL);
 	if (dbus_folder_hook_id == HOOK_NONE) {
-		g_warning("Failed to register folder update hook");
+		g_warning("failed to register folder update hook");
 		uninstall_dbus_status_handler();
 		return;
 	}
@@ -1079,10 +1131,6 @@ int main(int argc, char *argv[])
 	
 	if (cmd.exit)
 		return 0;
-#if !GLIB_CHECK_VERSION(2,32,0)
-	if (!g_thread_supported())
-		g_thread_init(NULL);
-#endif
 
 	reset_statistics();
 	
@@ -1142,10 +1190,6 @@ int main(int argc, char *argv[])
 	/* Create container for all the menus we will be adding */
 	MENUITEM_ADDUI("/", "Menus", NULL, GTK_UI_MANAGER_MENUBAR);
 
-	if (!g_thread_supported()) {
-		g_error(_("g_thread is not supported by glib.\n"));
-	}
-
 #ifdef G_OS_WIN32
 	CHDIR_EXEC_CODE_RETURN_VAL_IF_FAIL(get_home_dir(), 1, win32_close_log(););
 #else
@@ -1318,7 +1362,7 @@ int main(int argc, char *argv[])
 	}
 #endif
 	gtkut_widget_init();
-	stock_pixbuf_gdk(STOCK_PIXMAP_CLAWS_MAIL_ICON, &icon);
+	priv_pixbuf_gdk(PRIV_PIXMAP_CLAWS_MAIL_ICON, &icon);
 	gtk_window_set_default_icon(icon);
 
 	folderview_initialize();
@@ -1415,9 +1459,6 @@ int main(int argc, char *argv[])
 		}
 		if(!account_get_list()) {
 			exit_claws(mainwin);
-#ifdef G_OS_WIN32
-			win32_close_log();
-#endif
 			exit(1);
 		}
 		never_ran = TRUE;
@@ -1456,7 +1497,7 @@ int main(int argc, char *argv[])
 	}
 	/* make the crash-indicator file */
 	if (str_write_to_file("foo", get_crashfile_name(), FALSE) < 0) {
-		g_warning("Can't create the crash-indicator file.");
+		g_warning("can't create the crash-indicator file");
 	}
 
 	inc_autocheck_timer_init(mainwin);
@@ -1566,9 +1607,6 @@ int main(int argc, char *argv[])
 				   "external plugin. Please reinstall the "
 				   "plugin and try again."));
 			exit_claws(mainwin);
-#ifdef G_OS_WIN32
-			win32_close_log();
-#endif
 			exit(1);
 		}
 	}
@@ -1652,9 +1690,6 @@ int main(int argc, char *argv[])
 	uninstall_dbus_status_handler();
 	if(connection)
 		dbus_g_connection_unref(connection);
-#endif
-#ifdef G_OS_WIN32
-	win32_close_log();
 #endif
 	utils_free_regex();
 	exit_claws(mainwin);
@@ -1780,6 +1815,9 @@ static void exit_claws(MainWindow *mainwin)
 	gtkaspell_checkers_quit();
 #endif
 	plugin_unload_all("Common");
+#ifdef G_OS_WIN32
+	win32_close_log();
+#endif
 	claws_done();
 }
 
@@ -1789,10 +1827,9 @@ static void exit_claws(MainWindow *mainwin)
 		exit(1);	\
 	}
 
-static GString * parse_cmd_compose_from_file(const gchar *fn)
+static void parse_cmd_compose_from_file(const gchar *fn, GString *body)
 {
 	GString *headers = g_string_new(NULL);
-	GString *body = g_string_new(NULL);
 	gchar *to = NULL;
 	gchar *h;
 	gchar *v;
@@ -1849,8 +1886,6 @@ static GString * parse_cmd_compose_from_file(const gchar *fn)
 	/* append the remaining headers */
 	g_string_append(body, headers->str);
 	g_string_free(headers, TRUE);
-
-	return body;
 }
 
 #undef G_PRINT_EXIT
@@ -1867,6 +1902,8 @@ static void parse_cmd_opt_error(char *errstr, char* optstr)
 	exit(1);
 }
 
+static GString mailto; /* used to feed cmd.compose_mailto when --compose-from-file is used */
+
 static void parse_cmd_opt(int argc, char *argv[])
 {
 	AttachInfo *ainfo;
@@ -1885,9 +1922,9 @@ static void parse_cmd_opt(int argc, char *argv[])
 		    if (i+1 < argc) {
 				const gchar *p = argv[i+1];
 
-				GString *mailto = parse_cmd_compose_from_file(p);
+				parse_cmd_compose_from_file(p, &mailto);
 				cmd.compose = TRUE;
-				cmd.compose_mailto = mailto->str;
+				cmd.compose_mailto = mailto.str;
 				i++;
 		    } else {
                 parse_cmd_opt_error(_("Missing file argument for option %s"), argv[i]);
@@ -2314,6 +2351,7 @@ gchar *claws_get_socket_name(void)
 			/* old versions used a sock in $TMPDIR/claws-mail-$UID */
 			debug_print("Using legacy socket %s\n", socket_dir);
 			filename = g_strdup(socket_dir);
+			g_free(socket_dir);
 			return filename;
 		}
 
@@ -2503,7 +2541,7 @@ static gint prohibit_duplicate_launch(void)
 			buf[sizeof(buf) - 1] = '\0';
  			if (!STRNCMP(buf, ".\n")) break;
 			if (claws_fputs(buf, stdout) == EOF) {
-				g_warning("writing to stdout failed.");
+				g_warning("writing to stdout failed");
 				break;
 			}
  		}
@@ -2517,7 +2555,7 @@ static gint prohibit_duplicate_launch(void)
 			buf[sizeof(buf) - 1] = '\0';
  			if (!STRNCMP(buf, ".\n")) break;
 			if (claws_fputs(buf, stdout) == EOF) {
-				g_warning("writing to stdout failed.");
+				g_warning("writing to stdout failed");
 				break;
 			}
  		}
@@ -2540,7 +2578,7 @@ static gint prohibit_duplicate_launch(void)
 			buf[sizeof(buf) - 1] = '\0';
 			if (!STRNCMP(buf, ".\n")) break;
 			if (claws_fputs(buf, stdout) == EOF) {
-				g_warning("writing to stdout failed.");
+				g_warning("writing to stdout failed");
 				break;
 			}
 		}
@@ -2585,7 +2623,8 @@ static gint lock_socket_remove(void)
 #ifdef G_OS_UNIX
 	filename = claws_get_socket_name();
 	dirname = g_path_get_dirname(filename);
-	claws_unlink(filename);
+	if (claws_unlink(filename) < 0)
+                FILE_OP_ERROR(filename, "claws_unlink");
 	g_rmdir(dirname);
 	g_free(dirname);
 #endif
@@ -2892,12 +2931,14 @@ static void send_queue(void)
 	}
 }
 
+#ifndef G_OS_WIN32
 static void quit_signal_handler(int sig)
 {
 	debug_print("Quitting on signal %d\n", sig);
 
 	g_timeout_add(0, clean_quit, NULL);
 }
+#endif
 
 static void install_basic_sighandlers()
 {
