@@ -768,6 +768,16 @@ void cm_gdata_load_contacts_cache_from_file(void)
       }
       else {
         debug_print("Ignored contact without email address: %s\n", cached_contact->full_name ? cached_contact->full_name : "(null)");
+        /* Not added to list: return allocated memory */
+        if (cached_contact->full_name)
+            g_free(cached_contact->full_name);
+        if (cached_contact->given_name)
+            g_free(cached_contact->given_name);
+        if (cached_contact->family_name)
+            g_free(cached_contact->family_name);
+        if (cached_contact->address)
+            g_free(cached_contact->address);
+        g_free(cached_contact);
       }
     }
   }
