@@ -60,6 +60,9 @@ lh_widget::lh_widget()
 {
 	GtkWidget *item;
 
+	m_force_render = false;
+	m_blank = false;
+
 	/* scrolled window */
 	m_scrolled_window = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(m_scrolled_window),
@@ -195,11 +198,18 @@ void lh_widget::open_html(const gchar *contents)
 		adj = gtk_scrolled_window_get_vadjustment(
 				GTK_SCROLLED_WINDOW(m_scrolled_window));
 		gtk_adjustment_set_value(adj, 0.0);
-		redraw(false);
+		m_blank = false;
+		gtk_widget_queue_draw(m_drawing_area);
 	}
 	lh_widget_statusbar_pop();
 }
 
+void lh_widget::rerender()
+{
+	m_force_render = true;
+	gtk_widget_queue_draw(m_drawing_area);
+}
+
 void lh_widget::draw(cairo_t *cr)
 {
 	double x1, x2, y1, y2;
@@ -222,25 +232,23 @@ void lh_widget::draw(cairo_t *cr)
 	m_html->draw((litehtml::uint_ptr)cr, 0, 0, &pos);
 }
 
-void lh_widget::redraw(gboolean force_render)
+void lh_widget::redraw()
 {
 	GtkAllocation rect;
 	gint width;
 	GdkWindow *gdkwin;
 	cairo_t *cr;
 
-	paint_white();
-
 	if (m_html == NULL)
 		return;
 
 	/* Get width of the viewport. */
-	gdkwin = gtk_viewport_get_view_window(GTK_VIEWPORT(m_viewport));
-	width = gdk_window_get_width(gdkwin);
-	m_height = gdk_window_get_height(gdkwin);
+	gtk_widget_get_allocation(GTK_WIDGET(m_viewport), &rect);
+	width = rect.width;
+	m_height = rect.height;
 
 	/* If the available width has changed, rerender the HTML content. */
-	if (m_rendered_width != width || force_render) {
+	if (m_rendered_width != width || std::atomic_exchange(&m_force_render, false)) {
 		debug_print("lh_widget::redraw: width changed: %d != %d\n",
 				m_rendered_width, width);
 
@@ -266,33 +274,21 @@ void lh_widget::redraw(gboolean force_render)
 		return;
 	}
 	cr = gdk_cairo_create(GDK_DRAWABLE(gdkwin));
-	draw(cr);
-
-	cairo_destroy(cr);
-}
-
-void lh_widget::paint_white()
-{
-	GdkWindow *gdkwin = gtk_widget_get_window(m_drawing_area);
-	if (gdkwin == NULL) {
-		g_warning("lh_widget::clear: No GdkWindow to draw on!");
-		return;
+	if(!std::atomic_exchange(&m_blank, false)) {
+		draw(cr);
+	} else {
+		cairo_rectangle(cr, rect.x, rect.y, rect.width, rect.height);
+		cairo_set_source_rgb(cr, 255, 255, 255);
+		cairo_fill(cr);
 	}
-	cairo_t *cr = gdk_cairo_create(GDK_DRAWABLE(gdkwin));
-
-	/* Paint white background. */
-	gint width, height;
-	gdk_drawable_get_size(gdkwin, &width, &height);
-	cairo_rectangle(cr, 0, 0, width, height);
-	cairo_set_source_rgb(cr, 255, 255, 255);
-	cairo_fill(cr);
 
 	cairo_destroy(cr);
 }
+
 void lh_widget::clear()
 {
 	m_html = nullptr;
-	paint_white();
+	m_blank = true;
 	m_rendered_width = 0;
 	m_base_url.clear();
 	m_clicked_url.clear();
@@ -301,7 +297,6 @@ void lh_widget::clear()
 void lh_widget::set_cursor(const litehtml::tchar_t* cursor)
 {
 	litehtml::element::ptr over_el = m_html->over_element();
-	gint x, y;
 
 	if (m_showing_url &&
 			(over_el == NULL || over_el != m_over_element)) {
@@ -459,7 +454,7 @@ GdkPixbuf *lh_widget::get_local_image(const litehtml::tstring url) const
 
 			pixbuf = procmime_get_part_as_pixbuf(p, &error);
 			if (error != NULL) {
-				g_warning("Couldn't load image: %s\n", error->message);
+				g_warning("couldn't load image: %s", error->message);
 				g_error_free(error);
 				return NULL;
 			}
@@ -477,7 +472,7 @@ static gboolean expose_event_cb(GtkWidget *widget, GdkEvent *event,
 		gpointer user_data)
 {
 	lh_widget *w = (lh_widget *)user_data;
-	w->redraw(false);
+	w->redraw();
 	return FALSE;
 }
 
@@ -488,13 +483,13 @@ static gboolean button_press_event(GtkWidget *widget, GdkEventButton *event,
 	lh_widget *w = (lh_widget *)user_data;
 
 	if (w->m_html == NULL)
-		return false;
+		return FALSE;
 
 	//debug_print("lh_widget on_button_press_event\n");
 
 	if (event->type == GDK_2BUTTON_PRESS ||
 			event->type == GDK_3BUTTON_PRESS)
-		return true;
+		return TRUE;
 
 	/* Right-click */
 	if (event->button == 3) {
@@ -503,7 +498,7 @@ static gboolean button_press_event(GtkWidget *widget, GdkEventButton *event,
 		if (url != NULL)
 			w->popup_context_menu(url, event);
 
-		return true;
+		return TRUE;
 	}
 
 	if(w->m_html->on_lbutton_down((int) event->x, (int) event->y,
@@ -514,7 +509,7 @@ static gboolean button_press_event(GtkWidget *widget, GdkEventButton *event,
 		}
 	}
 	
-	return true;
+	return TRUE;
 }
 
 static gboolean motion_notify_event(GtkWidget *widget, GdkEventButton *event,
@@ -537,7 +532,7 @@ static gboolean motion_notify_event(GtkWidget *widget, GdkEventButton *event,
         }
 	}
 	
-	return true;
+	return TRUE;
 }
 
 static gboolean button_release_event(GtkWidget *widget, GdkEventButton *event,
@@ -545,20 +540,19 @@ static gboolean button_release_event(GtkWidget *widget, GdkEventButton *event,
 {
     litehtml::position::vector redraw_boxes;
     lh_widget *w = (lh_widget *)user_data;
-    GError* error = NULL;
 
 	if (w->m_html == NULL)
-		return false;
+		return FALSE;
 
 	//debug_print("lh_widget on_button_release_event\n");
 
 	if (event->type == GDK_2BUTTON_PRESS ||
 			event->type == GDK_3BUTTON_PRESS)
-		return true;
+		return TRUE;
 
 	/* Right-click */
 	if (event->button == 3)
-		return true;
+		return TRUE;
 
 	w->m_clicked_url.clear();
 
@@ -577,7 +571,7 @@ static gboolean button_release_event(GtkWidget *widget, GdkEventButton *event,
             open_uri(w->m_clicked_url.c_str(), prefs_common_get_uri_cmd());
     }
 
-	return true;
+	return TRUE;
 }
 
 static void open_link_cb(GtkMenuItem *item, gpointer user_data)
