@@ -455,7 +455,7 @@ void sieve_prefs_done(void)
 		return;
 
 	if (prefs_write_param(prefs, pref_file->fp) < 0) {
-		g_warning("failed to write ManageSieve Plugin configuration");
+		g_warning("failed to write ManageSieve plugin configuration");
 		prefs_file_close_revert(pref_file);
 		return;
 	}
@@ -586,9 +586,9 @@ void sieve_prefs_account_set_config(
 			config->host && config->host[0] ? config->host : "!",
 			config->use_port ? 'y' : 'n',
 			config->port,
-			config->tls_type,
-			config->auth,
-			config->auth_type,
+			(unsigned short)config->tls_type,
+			(unsigned short)config->auth,
+			(unsigned short)config->auth_type,
 			enc_userid ? enc_userid : "");
 
 	if (enc_userid)
