@@ -1,6 +1,6 @@
 /*
  * Claws Mail -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2016 Salvatore De Paolis & the Claws Mail Team
+ * Copyright (C) 1999-2021 the Claws Mail Team and Salvatore De Paolis
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -856,7 +856,7 @@ static void pdf_viewer_button_press_events_cb(GtkWidget *widget, GdkEventButton
 			dest = poppler_document_find_dest(
 					viewer->pdf_doc, viewer->link_action->goto_remote.dest->named_dest);
 			if (dest->type != POPPLER_DEST_XYZ) {
-				g_warning ("couldn't figure out link");
+				g_warning("couldn't figure out link");
 				poppler_dest_free(dest);
 				break;
 			}
@@ -1333,7 +1333,7 @@ static void pdf_viewer_update(MimeViewer *_viewer, gboolean reload_file, int pag
 				tmpfile = get_tmp_file();
 				cmdline = g_strdup_printf(
 					"gs -dSAFER -dCompatibilityLevel=1.2 -q -dNOPAUSE -dBATCH "
-					  "-sDEVICE=pdfwrite -sOutputFile=%s -c .setpdfwrite -f \"%s\"",
+					  "-sDEVICE=pdfwrite -sOutputFile=%s -f \"%s\"",
 					tmpfile, viewer->filename);
 				result = execute_command_line(cmdline, FALSE, NULL);
 				if (result == 0) {
@@ -1445,7 +1445,7 @@ static void pdf_viewer_update(MimeViewer *_viewer, gboolean reload_file, int pag
 		viewer->pdf_page = poppler_document_get_page(viewer->pdf_doc, page_num - 1);
 
 		if (viewer->pdf_page == NULL) {
-			g_warning("Page not found");
+			g_warning("page not found");
 			return;
 		}   
 
