@@ -1,6 +1,6 @@
 /*
  * Claws Mail -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 2001-2016 the Claws Mail team
+ * Copyright (C) 2001-2021 the Claws Mail team
  * This file (C) 2005-2015 Andrej Kacian <andrej@kacian.sk>
  *
  * - Plugin preferences
@@ -113,7 +113,7 @@ rssyl_prefs_cookies_browse_cb(GtkWidget* widget, gpointer data)
 
 	utf8_filename = g_filename_to_utf8(filename, -1, NULL, NULL, NULL);
 	if (!utf8_filename) {
-		g_warning("rssyl_prefs_cookies_browse_cb(): failed to convert character set.");
+		g_warning("rssyl_prefs_cookies_browse_cb(): failed to convert character set");
 		utf8_filename = g_strdup(filename);
 	}
 	gtk_entry_set_text(GTK_ENTRY(dest), utf8_filename);
@@ -167,7 +167,7 @@ static void create_rssyl_prefs_page(PrefsPage *page,
 
 	/* Whether to verify SSL peer certificate */
 	ssl_verify_peer = gtk_check_button_new_with_label(
-			_("Verify SSL/TLS certificates validity for new feeds"));
+			_("Verify TLS certificates validity for new feeds"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ssl_verify_peer),
 			rssyl_prefs.ssl_verify_peer);
 	gtk_box_pack_start(GTK_BOX(vbox2), ssl_verify_peer, FALSE, FALSE, 0);
@@ -244,7 +244,7 @@ static void save_rssyl_prefs(PrefsPage *page)
 				return;
 
 	if( prefs_write_param(param, pref_file->fp) < 0 ) {
-		g_warning("Failed to write RSSyl plugin configuration");
+		g_warning("failed to write RSSyl plugin configuration");
 		prefs_file_close_revert(pref_file);
 		return;
 	}
