@@ -361,8 +361,8 @@ static void prefs_matcher_models_create(void)
 
 	store = gtk_list_store_new(3, G_TYPE_STRING, G_TYPE_INT, G_TYPE_BOOLEAN);
 	COMBOBOX_ADD(store, _("bytes"), SIZE_UNIT_BYTES);
-	COMBOBOX_ADD(store, _("kilobytes"), SIZE_UNIT_KBYTES);
-	COMBOBOX_ADD(store, _("megabytes"), SIZE_UNIT_MBYTES);
+	COMBOBOX_ADD(store, _("kibibytes"), SIZE_UNIT_KBYTES);
+	COMBOBOX_ADD(store, _("mebibytes"), SIZE_UNIT_MBYTES);
 	matcher.model_size_units = GTK_TREE_MODEL(store);
 	
 	store = gtk_list_store_new(3, G_TYPE_STRING, G_TYPE_INT, G_TYPE_BOOLEAN);
@@ -1500,6 +1500,7 @@ static MatcherProp *prefs_matcher_dialog_to_matcher(void)
 	gboolean case_sensitive;
 	const gchar *header;
 	const gchar *expr;
+    	gboolean expr_to_free = FALSE;
 	gint value, sel;
 	gint year, month, day, hour, minute;
 
@@ -1578,6 +1579,7 @@ static MatcherProp *prefs_matcher_dialog_to_matcher(void)
 			alertpanel_error(_("Invalid hour."));
 			return NULL;
 		}
+		expr_to_free = TRUE;
 		break;
 
 	case CRITERIA_TEST:
@@ -1689,6 +1691,9 @@ static MatcherProp *prefs_matcher_dialog_to_matcher(void)
 	matcherprop = matcherprop_new(criteria, header, matchtype,
 				      expr, value);
 
+	if (expr_to_free)
+		g_free((gchar *)expr);
+
 	return matcherprop;
 }
 
@@ -2163,21 +2168,22 @@ static void prefs_matcher_ok(void)
 
 				if (!matcher_str || strcmp(matcher_str, str) != 0) {
 	                        	val = alertpanel(_("Entry not saved"),
-       		                        	 _("The entry was not saved.\nClose anyway?"),
+       		                        	 _("The entry was not saved. Close anyway?"),
                		                	 GTK_STOCK_CLOSE,
 						 _("_Continue editing"),
 						 NULL,
 						 ALERTFOCUS_SECOND);
 					if (G_ALERTDEFAULT != val) {
 						g_free(matcher_str);						 
-	        	                        g_free(str);
+						g_free(str);
+						matcherlist_free(matchers);
 						return;
 					}
 				}
 				g_free(matcher_str);
 			}
 		}
-                g_free(str);
+		g_free(str);
 		gtk_widget_hide(matcher.window);
 		gtk_window_set_modal(GTK_WINDOW(matcher.window), FALSE);
 		if (matchers_callback != NULL)
